/*****************************************************************************
 * recode.c
 * 2000-03-04 Dobrica Pavlinusic <dpavlin@rot13.org>
 *	first really working version which doesn't overwrite chars which are
 *	in PostgreSQL buffer (I should really read examples more carefully!)
 * 2000-03-17 dpavlin	added to88592 function
 * 2000-03-18 dpavlin	added initcap2 function
 *****************************************************************************/

#include <string.h>
#include <stdio.h>
#include "postgres.h"			/* for char16, etc. */
#include "utils/palloc.h"		/* for palloc */
#include "libpq-fe.h"			/* for TUPLE */
#include <stdio.h>
#include <ctype.h>

/* to1250 -- convert iso8859-2 chars to cp1250 */

text *
to1250(text *t)
{
	int32 i;
	char c;
	text *new_t = (text *) palloc(VARSIZE(t));
	memset(new_t, 0, VARSIZE(t));
	VARSIZE(new_t) = VARSIZE(t);

	for(i=0; i<=VARSIZE(t)-VARHDRSZ; i++) {
		c=VARDATA(t)[i];
		switch (c) {
			case (char)0xa9: c=(char)0x8a; break;
			case (char)0xb9: c=(char)0x9a; break;
			case (char)0xae: c=(char)0x8e; break;
			case (char)0xbe: c=(char)0x9e; break;
		}
		VARDATA(new_t)[i]=c;
	}

	return (new_t);
}

/* to88592 -- convert cp1250 to iso8859-2 chars */

text *
to88592(text *t)
{
	int32 i;
	char c;
	text *new_t = (text *) palloc(VARSIZE(t));
	memset(new_t, 0, VARSIZE(t));
	VARSIZE(new_t) = VARSIZE(t);

	for(i=0; i<=VARSIZE(t)-VARHDRSZ; i++) {
		c=VARDATA(t)[i];
		switch (c) {
			case (char)0x8a: c=(char)0xa9; break;
			case (char)0x9a: c=(char)0xb9; break;
			case (char)0x8e: c=(char)0xae; break;
			case (char)0x9e: c=(char)0xbe; break;
		}
		VARDATA(new_t)[i]=c;
	}

	return (new_t);
}

/* tocsz -- convert iso8859-2 or cp1250 to czs */

text *
toczs(text *t)
{
	int32 i;
	char c;
	text *new_t = (text *) palloc(VARSIZE(t));
	memset(new_t, 0, VARSIZE(t));
	VARSIZE(new_t) = VARSIZE(t);

	for(i=0; i<=VARSIZE(t)-VARHDRSZ; i++) {
		c=VARDATA(t)[i];
		switch (c) {
			/* iso8859-2 */
			case (char)0xa9: c=(char)'S'; break;
			case (char)0xb9: c=(char)'s'; break;
			case (char)0xd0: c=(char)'D'; break;
			case (char)0xf0: c=(char)'d'; break;
			case (char)0xc8: c=(char)'C'; break;
			case (char)0xe8: c=(char)'c'; break;
			case (char)0xc6: c=(char)'C'; break;
			case (char)0xe6: c=(char)'c'; break;
			case (char)0xae: c=(char)'Z'; break;
			case (char)0xbe: c=(char)'z'; break;
			/* cp1250 */
			case (char)0x8a: c=(char)'S'; break;
			case (char)0x9a: c=(char)'s'; break;
			case (char)0x8e: c=(char)'Z'; break;
			case (char)0x9e: c=(char)'z'; break;
		} 
		VARDATA(new_t)[i]=c;
	}

	return (new_t);
}

/* initcap2 -- initcap whith support for iso8859-2 or cp1250 */

text *
initcap2(text *t)
{
	int32 i;
	char c;
	int first = 1;	/* is this a first letter? */
	text *new_t = (text *) palloc(VARSIZE(t));
	memset(new_t, 0, VARSIZE(t));
	VARSIZE(new_t) = VARSIZE(t);

	for(i=0; i<=VARSIZE(t)-VARHDRSZ; i++) {
		c=VARDATA(t)[i];
		if (first) { 
			if (c >=(char)'a' && c <= (char)'z') { c-=32; }
			switch (c) {
				/* iso8859-2 */
				case (char)0xb9: c-=16; break;
				case (char)0xf0: c-=32; break;
				case (char)0xe8: c-=32; break;
				case (char)0xe6: c-=32; break;
				case (char)0xbe: c-=16; break;
				/* cp1250 */
				case (char)0x9a: c-=16; break;
				case (char)0x9e: c-=16; break;
			}
		} 
		if (!first) {
			if (c >='A' && c <= 'Z') { c+=32; }
			switch (c) {
				/* iso8859-2 */
				case (char)0xa9: c+=16; break;
				case (char)0xd0: c+=32; break;
				case (char)0xc8: c+=32; break;
				case (char)0xc6: c+=32; break;
				case (char)0xae: c+=16; break;
				/* cp1250 */
				case (char)0x8a: c+=16; break;
				case (char)0x8e: c+=16; break;
			}
		}
		if (c == ' ' || c == '-' || c =='.') first = 1; else first = 0;
		VARDATA(new_t)[i]=c;
	}

	return (new_t);
}

